/***************************************************************************
                          alignment.cpp  -  description
                             -------------------
    begin                : Thu Feb 20 2003
    copyright            : (C) 2003 by Vinh Le Sy and Arndt von Haeseler
    email                : vinh@cs.uni-duesseldorf.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <iostream>
#include <fstream>
#include <string.h>
#include "vec.h"
#include "ali.h"
#include "outstream.h"


using namespace std;

int Ali::nSeq_;
Vec<Seq> Ali::items_;
double **Ali::genDisMat_;

char Ali::fileName_[FILENAME_LEN];


//construction function
Ali::Ali () {
}

//clean all content of this class
void Ali::clean () {
  nSeq_ = 0;
  release ();
}

//this function is used for constructing this class
void Ali::doConstructor () {
  clean ();
}

//--------------------------------------------------------------------
//everything is inited here
void Ali::init () {
}

//get the order of the sequence whose name is input name
int Ali::getSeqNo (Vec<char> &name) {
  for (int count_ = 0; count_ < nSeq_; count_ ++)

    if (items_[count_].getName () == name) 
      return count_;

  return -1;
}

//--------------------------------------------------------------------
//get the name of the sequence at certain orther 
Vec<char> &Ali::getName (int ndNo) {
  static Vec<char> name_;
  name_.clean ();
  if (ndNo < nSeq_) {//it is a external node
    name_ = items_[ndNo].getName (); 
    return name_;
  }
  else { //it is an internal node   
    name_.convert (ndNo);
    return name_;
  }    
}

//--------------------------------------------------------------------
//get the name of the sequence at certain orther
void Ali::getName (int ndNo, Vec<char> &name) {
  if (ndNo < nSeq_) //it is a external node
    name = items_[ndNo].getName ();
}


//--------------------------------------------------------------------
//return the number sequences of this alignment
int Ali::getNSeq () {
  return nSeq_;
}


//--------------------------------------------------------------------
//read input from the file fileName
int Ali::readInputMatrix (ifstream &in, int isFirstTime) {

  int tmpNSeq_ = -1; 
  in >> tmpNSeq_;
  if (tmpNSeq_ == -1)
    return 0;
  else {
    nSeq_ = tmpNSeq_;
    if (isFirstTime == 1)
      locateMem ();
  }
  
  in.ignore (INFINITIVE, '\n');
  for (int rowNo_ = 0; rowNo_ < nSeq_; rowNo_ ++) {
    char name_[PHYLIP_SEQ_NAME_LEN + 1];
    in.read (name_, PHYLIP_SEQ_NAME_LEN);
    
    items_[rowNo_].setName (name_);
    for (int colNo_ = 0; colNo_ < nSeq_; colNo_ ++) {
      double dis_;
      in >> dis_;
      genDisMat_[rowNo_][colNo_] = dis_;
    }   
    in.ignore (INFINITIVE, '\n');  
  }
  
  return 1;
}

//--------------------------------------------------------------------
//read input from the file fileName
int Ali::readInput (const char *aliFileName, ifstream &aliFile) {

  aliFile.open (aliFileName);

  if (!aliFile) 
    return 0;
  memcpy (fileName_, aliFileName, strlen (aliFileName));

  int isFirstTime_ = 1;
  readInputMatrix (aliFile, isFirstTime_);
  
  
  return 1;
}

/*--------------------------------------------------------------------
sort sequences by their name, it serves for deburging
void Ali::sortSeq () {
  for (int seqNo1_ = 0; seqNo1_ < nSeq_; seqNo1_ ++) 
    for (int seqNo2_ = seqNo1_ + 1; seqNo2_ < nSeq_; seqNo2_ ++) 
      if (items_[seqNo1_].getName ().cmp (items_[seqNo2_].getName () ) == 1){
         Seq tmpSeq_;
         tmpSeq_ = items_[seqNo1_];
         items_[seqNo1_] = items_[seqNo2_];
         items_[seqNo2_] = tmpSeq_;
      }
}
*/

//============================================================
int Ali::checkSameName () {
  for (int seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++)
    for (int seqNo2_ = seqNo_ + 1; seqNo2_ < nSeq_; seqNo2_ ++)
      if (items_[seqNo_].getName () == items_[seqNo2_].getName ()) {
          std::cout << "The name " << items_[seqNo_].getName () << " appears more than one time in the distance matrix file!!!" << endl;
          std::cout << "Note that, all sequence names MUST have at least 11 characters as in PHYLIP format!!!";
          exit (EXIT_FAILURE);
      }
//  std::cout <<"no identical name" << endl;
  return 0;
}

//--------------------------------------------------------------------
//return the address of genetic distance matrix
double **Ali::getGenDis () {
  return genDisMat_;
}


void Ali::writeGenDis () {
  ofstream out;

  char distName_[FILENAME_LEN];
  strcpy (distName_, fileName_);
  char distTail_[FILENAME_LEN] = ".dist";
  strcat (distName_, distTail_);

  out.open (distName_);
  OutStream::writeGenDis (genDisMat_, nSeq_, out);
  out.close ();
  
}



/*--------------------------------------------------------------------
//return the state frequency list 
void Ali::getStateFrq (DVec4 &stateFrqArr) {
  stateFrqArr = stateFrqArr_;
}
*/

//set the memory for this class
void Ali::locateMem () {
  //do not need set limit again
  genDisMat_ = new double*[nSeq_];
  for (int row_ = 0; row_ < nSeq_; row_ ++ )
     genDisMat_[row_] = new double[nSeq_];


  items_.set (nSeq_, nSeq_);
  for (int seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++)
    items_[seqNo_].set (0, 0);
}

//--------------------------------------------------------------------
//release all memmory for this class
void Ali::release () {
  for (int row_ = 0; row_ < nSeq_; row_ ++ )
     delete [] genDisMat_[row_];
  delete [] genDisMat_;
  
  int realLimit_ = items_.getLimit ();

  items_.setSize (realLimit_);
  for (int count_ = 0; count_ < realLimit_; count_ ++)
    items_[count_].release ();  
  items_.release ();
}


//--------------------------------------------------------------------
//destruction function
Ali::~Ali () {
  release ();

//  std::cout << "this is the destructor of ali class " << endl;
}
