/***************************************************************************
                          alignment.h  -  description
                             -------------------
    begin                : Thu Feb 20 2003
    copyright            : (C) 2003 by Vinh Le Sy and Arndt von Haeseler
    email                : vinh@cs.uni-duesseldorf.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ALI_H
#define ALI_H

#include <iostream>
#include <fstream>

#include "vec.h"
#include "seq.h"
#include "mat.h"

using namespace std;

/*
The Alignment class contains all information and mothods for DNA alignment and Protein alignment,
Note: The Alignment class does not inherit from Sequence class
...Being specified more
*/
class Ali
{
public :
static char fileName_[FILENAME_LEN];

//construction function to init data
Ali ();
//use this function for constructing this class
static void doConstructor ();


//clean all content of this class
static void clean ();

//everything is inited here
static void init ();

//return the number sequences of this alignment
static int getNSeq ();


//return the sequence indexth
static Seq &getSeq (const int index);

//sort sequences by their name, it serves for deburging
//static void sortSeq ();


static int readInputMatrix (std::ifstream &in, int isFirstTime);

//read input from the file fileName
static int readInput (const char *aliFileName, ifstream &aliFile);

//return the address of genetic distance matrix
static double **getGenDis ();


//--------------------------------------------------------------------
//write all information of this alignment 
template <class Tout>
static Tout &write (Tout &out) {
  out << nSeq_ << endl;
  for (int seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++)
    items_[seqNo_].write (out);
  out << "=========end of this alignment====================================" << endl;
  return out;

}

//--------------------------------------------------------------------
//write all information of this alignment 
static void write (const char *fileName) {
  std::ofstream out_;
  out_.open (fileName);
  out_ << nSeq_ <<  endl;
  for (int seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++)
    items_[seqNo_].write (out_);

  out_.close ();
}

//get the name of the sequence at certain ndNo
static Vec<char> &getName (int ndNo);
//get the name of the sequence at certain ndNo
static void getName (int ndNo, Vec<char> &name);

//get the order of the sequence whose name is input name
static int getSeqNo (Vec<char> &name);


static int checkSameName ();

static void writeGenDis ();


//destruction function to release data
~Ali ();

/**************************************************************************
**************************************************************************
**************************************************************************
**************************************************************************/
private :
//the number of sequence of this alignment, we do not use it, instead of using nActSeq
static int nSeq_;

//the alignment information
static Vec<Seq> items_;

//disMat_[i][j] is the distance between sequence i and sequence j
static double **genDisMat_;




//set the memory for this class
static void locateMem ();

//release all memmory for this class
static void release ();


}; //end class ali
#endif
