#include "rootedtree.h"
#include "ali.h"
#include "outstream.h"

RootedTree::RootedTree () {
  rootNo_ = -1;
  lChi_ = 0;  
  rChi_ = 0;
  isLea_ = -1;
  faBrLen_ = -1.0;

  isNewTree_ = 1;
}

void RootedTree::init (int seqNo) {
  rootNo_ = seqNo;
  active_ = 1;
  neaSibNo_ = -1;
  neaSibDis_ = -INFINITIVE;

  
  leaArr_.set (1, 0);
  leaArr_ += seqNo;
  isLea_ = 1;

  repLeaArr_.set (1, 0);
  repLeaArr_ += seqNo;
  isNewTree_ = 0;
}


void RootedTree::turnOff () {
  active_ = 0;
}

void RootedTree::turnOn () {
  active_ = 1;
}

int RootedTree::isActive () {
  return active_;
}

int RootedTree::isLea () {
  return isLea_;
}

void RootedTree::clean () {
  active_ = 0;
  rootNo_ = -1;
  isLea_ = -1;

  lChi_ = 0;

  rChi_ = 0;

  neaSibNo_ = -1;
  neaSibDis_ = -INFINITIVE;
}


void RootedTree::setFaBrLen (double faBrLen) {
  faBrLen_ = faBrLen;
}

double RootedTree::getFaBrLen () {
  return faBrLen_;
}

Vec<int> *RootedTree::getRepLea () {
  return &repLeaArr_;
}

Vec<int> *RootedTree::getRepOutGrpNd () {
  return &repOutGrpNdArr_;
}

Vec<double> *RootedTree::getRepDis () {
  return &repDisArr_;
}

int RootedTree::getRoot () {
  return rootNo_;
}

int RootedTree::getNeaSibNo () {
  return neaSibNo_;
}

double RootedTree::getNeaSibDis () {
  return neaSibDis_;
}

void RootedTree::setNeaSib (int neaSibNo, double neaSibDis) {
  neaSibNo_ = neaSibNo;
  neaSibDis_ = neaSibDis;
}

RootedTree *RootedTree::getLChi () {
  return lChi_;
}

RootedTree *RootedTree::getRChi () {
  return rChi_;
}



Vec<int> *RootedTree::getLea () {
  return &leaArr_;
}

/*=========================================================================
void RootedTree::cmpTripletDis (int outGrpNdNo, int lRepLea, int rRepLea, double &disOx, double &disLx, double &disRx) {
  double **genDisMat_;
  genDisMat_ = Ali::getGenDis ();
  double disOL_ = genDisMat_[outGrpNdNo][lRepLea];
  double disOR_ = genDisMat_[outGrpNdNo][rRepLea];
  double disLR_ = genDisMat_[lRepLea][rRepLea];
  double halfSum_ = (disOL_ + disOR_ + disLR_) / 2.0;
  disOx = halfSum_ - disLR_;
  disLx = halfSum_ - disOR_;
  disRx = halfSum_ - disOL_;
}

*/

//=========================================================================
void RootedTree::cmpNewTripletDis (int outGrpNdNo, int lRepLea, int rRepLea, double &disOx, double &disLx, double &disRx) {
  double **genDisMat_;
  genDisMat_ = Ali::getGenDis ();
  double disOL_ = genDisMat_[outGrpNdNo][lRepLea];
  double disOR_ = genDisMat_[outGrpNdNo][rRepLea];
  double disLR_ = genDisMat_[lRepLea][rRepLea];
  double halfSum_ = (disOL_ + disOR_ + disLR_) / 2.0;
  disOx = halfSum_ - disLR_;
  disLx = halfSum_ - disOR_;
  disRx = halfSum_ - disOL_;
}



//======================================================
int RootedTree::findCloOutGrpNd () {
  int markArr_[MAX_NUM_SEQ];
  int seqNo_;
  int nSeq_ = Ali::getNSeq ();

  for (seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++) 
    markArr_[seqNo_] = 0;
  
  int nCurLea_ = leaArr_.getSize ();
  for (int count_ = 0; count_ < nCurLea_; count_ ++)
    markArr_[ leaArr_[count_] ] = 1;

  int repLeaNo_ = leaArr_[0]; 

  int repOutGrpNdNo_ = -1;
  double minDis_ = INFINITIVE;
 
  double **genDisMat_;
  genDisMat_ = Ali::getGenDis ();

  for (seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++)
    if (markArr_[ seqNo_ ] == 0 && genDisMat_[seqNo_][repLeaNo_] < minDis_) {
      minDis_ = genDisMat_[seqNo_][repLeaNo_];
      repOutGrpNdNo_ = seqNo_;
    }

  return repOutGrpNdNo_;
}

//======================================================
int RootedTree::findCloOutGrpNd (int nRepLea) {
  int markArr_[MAX_NUM_SEQ];
  int seqNo_;
  int nSeq_ = Ali::getNSeq ();

  for (seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++) 
    markArr_[seqNo_] = 0;
  
  int nCurLea_ = leaArr_.getSize ();
  for (int count_ = 0; count_ < nCurLea_; count_ ++)
    markArr_[ leaArr_[count_] ] = 1;

  double **genDisMat_;
  genDisMat_ = Ali::getGenDis ();
 
  int repLeaNo_ = leaArr_[0];

  repOutGrpNdArr_.set (nRepLea + 1, 0);
  int nCurOutGrpNd_ = 0;

  for (seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++)
    if (markArr_[ seqNo_ ] == 0) {
       int count_;
       for (count_ = 0; count_ < nCurOutGrpNd_; count_ ++)
          if (genDisMat_[seqNo_][repLeaNo_] < genDisMat_[ repOutGrpNdArr_[count_]] [repLeaNo_]) 
             break;

      
       repOutGrpNdArr_.insert (count_, seqNo_);
       if (nCurOutGrpNd_ == nRepLea) 
           repOutGrpNdArr_.delLastItem ();
       else
           nCurOutGrpNd_ ++;
    }

  return nCurOutGrpNd_;
}

//======================================================
int RootedTree::createCloOutGrpNd (Vec<int> &canNdArr0, Vec<int> &canNdArr1, int nRepLea) {
  double **genDisMat_;
  genDisMat_ = Ali::getGenDis ();
 
  int repLeaNo_ = repLeaArr_[0];

  repOutGrpNdArr_.set (nRepLea + 1, 0);
  int nCanNd0_ = canNdArr0.getSize ();
  int nCanNd1_ = canNdArr1.getSize ();
  int nCanNd_ = Utl::getMin (nCanNd0_ + nCanNd1_ , nRepLea);  
  int pos0_ = 0;
  int pos1_ = 0;  
  for (int count_ = 0; count_ < nCanNd_; count_ ++) {
    int status_ = -1;

    if (pos1_ == nCanNd1_) 
      status_ = 0;
    if (pos0_ == nCanNd0_)
      status_ = 1;
    if (status_ == -1)
      if (genDisMat_[repLeaNo_][ canNdArr0[pos0_]] < genDisMat_[repLeaNo_][ canNdArr1[pos1_]])
        status_ = 0;
      else
        status_ = 1;

    if (status_ == 0) {
       repOutGrpNdArr_ += canNdArr0[pos0_];
       pos0_ ++;
    } else {
       repOutGrpNdArr_ += canNdArr1[pos1_];
       pos1_ ++;
    }
  }
  return nCanNd_;
}

//===================================================================
int RootedTree::createRepLea (Vec<int> &canLeaArr0, Vec<int> &canLeaArr1, int nRepLea, int outGrpNdNo) {
  double **genDisMat_;
  genDisMat_ = Ali::getGenDis ();
 
  if (outGrpNdNo == -1) 
    outGrpNdNo = findCloOutGrpNd ();

  repLeaArr_.set (nRepLea + 1, 0);

  int nCanLea0_ = canLeaArr0.getSize ();
  int nCanLea1_ = canLeaArr1.getSize ();
  int nCanLea_ = Utl::getMin (nCanLea0_ + nCanLea1_ , nRepLea);  
  int pos0_ = 0;
  int pos1_ = 0;  

  for (int count_ = 0; count_ < nCanLea_; count_ ++) {
    int status_ = -1;
    if (pos1_ == nCanLea1_) 
      status_ = 0;
    if (pos0_ == nCanLea0_)
      status_ = 1;
   if (status_ == -1)
      if ( genDisMat_[outGrpNdNo][ canLeaArr0[pos0_]] < genDisMat_[outGrpNdNo][ canLeaArr1[pos1_]])
        status_ = 0;
      else
        status_ = 1;

    if (status_ == 0) {
       repLeaArr_ += canLeaArr0[pos0_];
       pos0_ ++;
    } else {
       repLeaArr_ += canLeaArr1[pos1_];
       pos1_ ++;
    }
  }
//  for (int count_ = 0; count_ < nCanLea_; count_ ++)
  //  std::cout << '(' << repLeaArr_[count_] << " ,  " << genDisMat_[outGrpNdNo][repLeaArr_[count_]] << ")       " << endl;
//  std::cout << endl;
  return nCanLea_;
}

//======================================================
int RootedTree::createRepLea (int nRepLea) {
  int markArr_[MAX_NUM_SEQ];
  int seqNo_;
  int nSeq_ = Ali::getNSeq ();

  for (seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++) 
    markArr_[seqNo_] = 0;
  
  int nCurLea_ = leaArr_.getSize ();
  for (int count_ = 0; count_ < nCurLea_; count_ ++)
    markArr_[ leaArr_[count_] ] = 1;

  double **genDisMat_;
  genDisMat_ = Ali::getGenDis ();
 
  int outGrpNdNo_ = findCloOutGrpNd ();

  repLeaArr_.set (nRepLea + 1, 0);
  int nCurRepLea_ = 0;

  for (seqNo_ = 0; seqNo_ < nSeq_; seqNo_ ++)
    if (markArr_[ seqNo_ ] == 1) {
       int count_;
       for (count_ = 0; count_ < nCurRepLea_; count_ ++)
          if (genDisMat_[seqNo_][outGrpNdNo_] < genDisMat_[ repLeaArr_[count_]] [outGrpNdNo_]) 

             break;

      
       repLeaArr_.insert (count_, seqNo_);
       if (nCurRepLea_ == nRepLea) 
           repLeaArr_.delLastItem ();
       else
           nCurRepLea_ ++;
    }

//  for (int count_ = 0; count_ < nCurRepLea_; count_ ++)
//    std::cout << '(' << repLeaArr_[count_] << " ,  " << genDisMat_[outGrpNdNo_][repLeaArr_[count_]] << ')       ' << endl;
//  std::cout << endl;
  return nCurRepLea_;
}

/*-------------------------------------------------------------
//compute the observed distance among the outgrpnd and two representive colections
double RootedTree::cmpOutGrpDis (Vec<int> &outGrpArr, Vec<int> &lRepLeaArr, Vec<int> &rRepLeaArr, Vec<double> &outGrpDisArr) {
  int nLRepLea_ = lRepLeaArr.getSize ();
  int nRRepLea_ = rRepLeaArr.getSize ();
  int nOutGrpLea_ = outGrpArr.getSize ();
  double **genDisMat_;
  genDisMat_ = Ali::getGenDis ();

  double totalOutGrpDis_ = 0.0;
  outGrpDisArr.set (nOutGrpLea_, nOutGrpLea_);

  for (int outCount_ = 0; outCount_ < nOutGrpLea_; outCount_ ++) {
     int outGrpNdNo_ = outGrpArr[outCount_];

     double totalWei_ = 0.0;
     double totalDis_ = 0.0;
     for (int lCount_ = 0; lCount_ < nLRepLea_; lCount_ ++) 
        for (int rCount_ = 0; rCount_ < nRRepLea_; rCount_ ++) {
           int lRepLeaNo_ = lRepLeaArr[lCount_];
           int rRepLeaNo_ = rRepLeaArr[rCount_];
           double dis12_ = genDisMat_[outGrpNdNo_][lRepLeaNo_];
           double dis13_ = genDisMat_[outGrpNdNo_][rRepLeaNo_];
           double dis23_ = genDisMat_[lRepLeaNo_][rRepLeaNo_];
           double sumDis_ = (dis12_ + dis13_ + dis23_) ;

           double outGrpDis_ = sumDis_ / 2  -  dis23_;

           double wei_ = 1.0 / sumDis_ ;

           totalDis_ += wei_ * outGrpDis_;
           totalWei_ += wei_;
       }

     outGrpDisArr[outCount_] = totalDis_ / totalWei_;
     totalOutGrpDis_ += outGrpDisArr[outCount_];
  }
  
  return  totalOutGrpDis_;
}    
*/
//-------------------------------------------------------------
//compute the observed distance among the outgrpnd and two representive colections
double RootedTree::cmpOutGrpDis (Vec<int> &outGrpArr, Vec<int> &lRepLeaArr, Vec<int> &rRepLeaArr) {
  int nLRepLea_ = lRepLeaArr.getSize ();
  int nRRepLea_ = rRepLeaArr.getSize ();
  int nOutGrpLea_ = outGrpArr.getSize ();
  double **genDisMat_;
  genDisMat_ = Ali::getGenDis ();

  double totalOutGrpDis_ = 0.0;

  for (int outCount_ = 0; outCount_ < nOutGrpLea_; outCount_ ++) {
     int outGrpNdNo_ = outGrpArr[outCount_];

     for (int lCount_ = 0; lCount_ < nLRepLea_; lCount_ ++) 
        for (int rCount_ = 0; rCount_ < nRRepLea_; rCount_ ++) {
           int lRepLeaNo_ = lRepLeaArr[lCount_];
           int rRepLeaNo_ = rRepLeaArr[rCount_];
           double dis12_ = genDisMat_[outGrpNdNo_][lRepLeaNo_];
           double dis13_ = genDisMat_[outGrpNdNo_][rRepLeaNo_];
           double dis23_ = genDisMat_[lRepLeaNo_][rRepLeaNo_];

           totalOutGrpDis_ += (dis12_ + dis13_ -  dis23_)  / 2.0;
       }

  }
  
  return  totalOutGrpDis_ / (nLRepLea_ * nRRepLea_ * nOutGrpLea_);
}    


/*=======================================================
//-------------------------------------------------------------
//compute the observed distance among the outgrpnd and two representive colections
void RootedTree::cmpOutGrpDis (Vec<double> &outGrpDisArr0, Vec<double> &outGrpDisArr1, Vec<double> &outGrpDisArr2, 
                                                    double &totalOutGrpDis0,
                                                    double &totalOutGrpDis1,

                                                    double &totalOutGrpDis2) {
  int nOutGrpNd_ = outGrpDisArr0.getSize ();
  totalOutGrpDis0 = 0.0;
  totalOutGrpDis1 = 0.0;
  totalOutGrpDis2 = 0.0;

  double totalWei_ = 0.0;
  for (int count_ = 0; count_ < nOutGrpNd_; count_ ++) {
      double maxDis_ = Utl::getMax (outGrpDisArr0[count_], outGrpDisArr1[count_], outGrpDisArr2[count_]);
      double wei_ = exp (- maxDis_);
      totalWei_ += wei_;
      totalOutGrpDis1 += (outGrpDisArr1[count_] - outGrpDisArr0[count_]) * wei_;
      totalOutGrpDis2 += (outGrpDisArr2[count_] - outGrpDisArr0[count_]) * wei_;
  }
 
  totalOutGrpDis0 = 0.0;
  totalOutGrpDis1 /= totalWei_;
  totalOutGrpDis2 /= totalWei_;
}    
*/

//=============================================================
RootedTree *RootedTree::getNeiTree (RootedTree *chi) {
   if (lChi_ == chi)
      return rChi_;
   else
      return lChi_;
}

void RootedTree::changeChi (RootedTree *oldChi, RootedTree *newChi) {
   if (lChi_ == oldChi)
      lChi_ = newChi;
   else
      rChi_ = newChi;
}

//=============================================================
//=============================================================
//=============================================================
int RootedTree::correct (int nRepLea) { 
   findCloOutGrpNd (nRepLea);
   //std::cout << rootNo_ << "/" << nRepLea << endl;
   lChi_->correct (*this, nRepLea);
   rChi_->correct (*this, nRepLea);
   return 1;
}
//=============================================================
//=============================================================
//=============================================================
int RootedTree::correct (RootedTree &faTree, int nRepLea) {
   if (isLea () == 1)
      return 1;

  Vec<int> *faRepOutGrpNdArr_;
  faRepOutGrpNdArr_ = faTree.getRepOutGrpNd();
  int bestFaRepOutGrpNdNo_ = (*faRepOutGrpNdArr_)[0];
  
  RootedTree *neiTree_;
  neiTree_ = faTree.getNeiTree (this);

//  int neiRootNo_ = neiTree_->getRoot ();

  double leftRightDis_ = cmpOutGrpDis (*faRepOutGrpNdArr_, *lChi_->getRepLea (), *rChi_->getRepLea ());
  double leftNeiDis_ = cmpOutGrpDis (*faRepOutGrpNdArr_, *lChi_->getRepLea () , *neiTree_->getRepLea ());
  double rightNeiDis_ = cmpOutGrpDis (*faRepOutGrpNdArr_, *rChi_->getRepLea () , *neiTree_->getRepLea ());

  int isNNI_ = 0;
  //left tree and neighbor tree are true neighbors
  if (leftNeiDis_ > leftRightDis_ && leftNeiDis_ > rightNeiDis_) {
     RootedTree *oldRChi_;
     oldRChi_ = rChi_;
     rChi_ = neiTree_;
     faTree.changeChi (neiTree_, oldRChi_);
     createCloOutGrpNd (*faRepOutGrpNdArr_, *oldRChi_->getRepLea (), nRepLea);
     createRepLea (*lChi_->getRepLea (), *rChi_->getRepLea (), nRepLea, bestFaRepOutGrpNdNo_);
     isNNI_ = 1;
  }

  if (rightNeiDis_ > leftRightDis_ && rightNeiDis_ > leftNeiDis_) {
     RootedTree *oldLChi_;
     oldLChi_ = lChi_;
     lChi_ = neiTree_;
     faTree.changeChi (neiTree_, oldLChi_);
     createCloOutGrpNd (*faRepOutGrpNdArr_, *oldLChi_->getRepLea (), nRepLea);
     createRepLea (*lChi_->getRepLea (), *rChi_->getRepLea (), nRepLea, bestFaRepOutGrpNdNo_);
     isNNI_ = 1;
  }

  if (isNNI_ == 1) {
      lChi_->correct (*this, nRepLea);
      rChi_->correct (*this, nRepLea);
  }

  return isNNI_;                                                     
}

//==================================================================
void RootedTree::createLea () {  
  Vec<int> *lLeaArr_;
  lLeaArr_ = lChi_->getLea ();
   
  Vec<int> *rLeaArr_;
  rLeaArr_ = rChi_->getLea ();

  int  lNLea_ = lLeaArr_->getSize ();
  int  rNLea_ = rLeaArr_->getSize ();
  
  leaArr_.set(lNLea_ + rNLea_, 0);

  for (int lLeaNo_ = 0; lLeaNo_ < lNLea_; lLeaNo_ ++)
    leaArr_ += (*lLeaArr_)[lLeaNo_];

  for (int rLeaNo_ = 0; rLeaNo_ < rNLea_; rLeaNo_ ++)
    leaArr_ += (*rLeaArr_)[rLeaNo_];
}


void RootedTree::mergeTwoTree (int outGrpNdNo, int rootNo, int nRepLea, RootedTree *lTree, RootedTree *rTree) {
  rootNo_ = rootNo;

  
  lChi_ = lTree;
  rChi_ = rTree;

  lChi_->turnOff ();
  rChi_->turnOff ();

  isLea_ = 0;
 
  if (isNewTree_ == 1) {
     createLea ();
     isNewTree_ = 0;
  }

  createRepLea (*lChi_->getRepLea (), *rChi_->getRepLea (), nRepLea, outGrpNdNo);


  neaSibNo_ = -1;
  neaSibDis_ = -INFINITIVE;

}


//=======================================================
Vec<char> *RootedTree::getNwTree () {
  return &nwTree_;
}

void RootedTree::createNwTree () {
  char colon_ = ':';
  char openBracket_ = '(';
  char comma_ = ',';
  char closeBracket_ = ')';

  Vec<char> tmpNwTree_;
  if (isLea () == 1) {
    tmpNwTree_.set (MAX_SEQ_NAME + BR_LEN_PRECISION + 2, 0);
    Ali::getName (rootNo_, tmpNwTree_);
  } else {
    lChi_->createNwTree ();
    rChi_->createNwTree ();

    tmpNwTree_.set (MAX_NEWICK_TREE_LEN, 0);
    tmpNwTree_ += openBracket_;
    tmpNwTree_ += *lChi_->getNwTree ();
    tmpNwTree_ += comma_;
    tmpNwTree_ += *rChi_->getNwTree ();
    tmpNwTree_ += closeBracket_;
  }
     
  Vec<char> sFaBrLen_ (BR_LEN_PRECISION + 1);
  sFaBrLen_.convert (faBrLen_, BR_LEN_PRECISION);
  tmpNwTree_+= colon_;
  tmpNwTree_ += sFaBrLen_; 
  nwTree_ = tmpNwTree_;
}


//-------------------------------------------------------------
//compute the observed distance among the outgrpnd and two representive colections
void RootedTree::cmpSTCDis (double &outDis, double &lDis, double &rDis) {

   Vec<int> *lRepLeaArr_;
  lRepLeaArr_ = lChi_->getRepLea ();

  Vec<double> *lRepDisArr_;
  lRepDisArr_ = lChi_->getRepDis ();

  Vec<int> *rRepLeaArr_;
  rRepLeaArr_ = rChi_->getRepLea ();

  Vec<double> *rRepDisArr_;
  rRepDisArr_ = rChi_->getRepDis ();


  int nLRepLea_ = lRepLeaArr_->getSize ();
  int nRRepLea_ = rRepLeaArr_->getSize ();
  int nOutGrpLea_ = repOutGrpNdArr_.getSize ();

  double **genDisMat_;
  genDisMat_ = Ali::getGenDis ();

  outDis = 0.0;
  lDis = 0.0;
  rDis = 0.0;

  for (int outCount_ = 0; outCount_ < nOutGrpLea_; outCount_ ++) {
     int outGrpNdNo_ = repOutGrpNdArr_[outCount_];

     for (int lCount_ = 0; lCount_ < nLRepLea_; lCount_ ++) 
        for (int rCount_ = 0; rCount_ < nRRepLea_; rCount_ ++) {
           int lRepLeaNo_ = (*lRepLeaArr_)[lCount_];
           int rRepLeaNo_ = (*rRepLeaArr_)[rCount_];
           double disOL_ = genDisMat_[outGrpNdNo_][lRepLeaNo_] - (*lRepDisArr_)[lCount_];
           double disOR_ = genDisMat_[outGrpNdNo_][rRepLeaNo_] - (*rRepDisArr_)[rCount_];
           double disLR_ = genDisMat_[lRepLeaNo_][rRepLeaNo_] - (*lRepDisArr_)[lCount_] - (*rRepDisArr_)[rCount_];

           double halfSumDis_ = (disOL_ + disOR_ + disLR_) / 2.0; 

           outDis += halfSumDis_ - disLR_;
           lDis += halfSumDis_ - disOR_;
           rDis += halfSumDis_ - disOL_;

       }
  }

  int nShortestTriplet_ = nOutGrpLea_ * nLRepLea_ * nRRepLea_;
  outDis /= nShortestTriplet_;
  lDis /= nShortestTriplet_;
  rDis /= nShortestTriplet_;
}    


//====================================================================
//====================================================================
//====================================================================
//====================================================================
//===================================================================
void RootedTree::createRepLeaDis (int nRepLea) {

  Vec<int> *lRepLeaArr_;
  lRepLeaArr_ = lChi_->getRepLea ();

  Vec<double> *lRepDisArr_;
  lRepDisArr_ = lChi_->getRepDis ();

  Vec<int> *rRepLeaArr_;
  rRepLeaArr_ = rChi_->getRepLea ();

  Vec<double> *rRepDisArr_;
  rRepDisArr_ = rChi_->getRepDis ();

  double lBrLen_ = lChi_->getFaBrLen ();
  double rBrLen_ = rChi_->getFaBrLen ();

  int nLRepLea_ = lRepLeaArr_->getSize ();
  int nRRepLea_ = rRepLeaArr_->getSize ();
  int nCanLea_ = Utl::getMin (nLRepLea_ + nRRepLea_, nRepLea);  
  
  repLeaArr_.set (nCanLea_, 0);
  repDisArr_.set (nCanLea_, nCanLea_);


  
  int pos0_ = 0;
  int pos1_ = 0;  

  for (int count_ = 0; count_ < nCanLea_; count_ ++) {
    int status_ = -1;
    if (pos0_ == nLRepLea_)
      status_ = 1;
    if (pos1_ == nRRepLea_) 
      status_ = 0;
   if (status_ == -1)
      if (lBrLen_ + (*lRepDisArr_)[pos0_] <= rBrLen_ + (*rRepDisArr_)[pos1_])
        status_ = 0;
      else
        status_ = 1;

    if (status_ == 0) {
       repLeaArr_ += (*lRepLeaArr_)[pos0_];
       repDisArr_[count_] = lBrLen_ + (*lRepDisArr_)[pos0_];
       pos0_ ++;
    } else {
       repLeaArr_ += (*rRepLeaArr_)[pos1_];
       repDisArr_[count_] = rBrLen_ + (*rRepDisArr_)[pos1_];
       pos1_ ++;
    }
  }

}

//====================================================================
//====================================================================
//====================================================================
//====================================================================
void RootedTree::createBrLen (int nRepLea) {
  if (isLea () == 1) {
     repLeaArr_.set (1, 0);
     repLeaArr_ += rootNo_;
     
     repDisArr_.set (1, 1);
     repDisArr_[0] = 0.0;
  } else {
     lChi_->createBrLen (nRepLea);
     rChi_->createBrLen (nRepLea);
//     int cloOutGrpNdNo_ = findCloOutGrpNd ();

     Vec<int> *lRepLeaArr_;
     lRepLeaArr_ = lChi_->getRepLea ();

     Vec<int> *rRepLeaArr_;
     rRepLeaArr_ = rChi_->getRepLea ();


     findCloOutGrpNd (nRepLea);

     double disOx_, disLx_, disRx_;
     cmpSTCDis (disOx_, disLx_, disRx_); 

     
     double lBrLen_ = Utl::getMax (MIN_BR_LEN, disLx_);
     double rBrLen_ = Utl::getMax (MIN_BR_LEN, disRx_);

     lChi_->setFaBrLen (lBrLen_);
     rChi_->setFaBrLen (rBrLen_);
     faBrLen_ = Utl::getMax (MIN_BR_LEN, disOx_);

     createRepLeaDis (nRepLea);
  }
}

//====================================================================
//====================================================================
//====================================================================
void RootedTree::reCreateLea () {
  if (isLea_ != 1) {
     lChi_->reCreateLea ();
     rChi_->reCreateLea ();
     createLea ();
  }   
}

//==========================================
void RootedTree::createNwTree (int outGrpNdNo, int nRepLea, Vec<char> &nwTree) {
  reCreateLea ();
                                                                                                                                                   
  createBrLen (nRepLea);

  char colon_ = ':';
  char openBracket_ = '(';
  char comma_ = ',';
  char closeBracket_ = ')';
  char semiColon_ = ';';

  lChi_->createNwTree ();
  rChi_->createNwTree ();
  nwTree_.set (MAX_NEWICK_TREE_LEN, 0);
  nwTree_ += openBracket_;
  nwTree_ += *lChi_->getNwTree ();
  nwTree_ += comma_;
  nwTree_ += *rChi_->getNwTree ();

  Vec<char> outGrpName_;
  Ali::getName (outGrpNdNo, outGrpName_);

  Vec<char> sOutGrpBrLen_;
  sOutGrpBrLen_.convert (faBrLen_, BR_LEN_PRECISION);

  nwTree_ += comma_;
  nwTree_ += outGrpName_;
  nwTree_ += colon_;
  nwTree_ += sOutGrpBrLen_;
  nwTree_ += closeBracket_;
  nwTree_ += semiColon_;

  nwTree = nwTree_;
}

RootedTree::~RootedTree () {
}

